/**
 * Accessibar - optionsDialog.js
 *
 * This file provides functionality for the options dialog whose interface
 * is described in optionsDialog.xul; and the option frames whose interfaces
 * are described in optionsKeyboard.xul and optionsReader.xul.
 *
 * @author Edan Kemelman <edankeme@yahoo.com>
 * @version 0.7.1
 */

// The following arrays will store preference values for selections on the options page
// interface that have yet to be saved in the user's preferences. 
var xsiBoolOpts = new Array();
var xsiIntOpts = new Array();
var xsiStrOpts = new Array();

// Default values for the various voice settings
const VOICE_DEFAULT = "kevin16"
const VOLUME_DEFAULT = 10;
const SPEED_DEFAULT = 150;
const PITCH_DEFAULT = 100;
const RANGE_DEFAULT = 15;

/**
 * This function handles <code>onselect</code> events on the <code>listbox</code> listing
 * the various option pages that can be chosen to be displayed. It will change the options
 * page displayed in the <code>iframe</code> section of the optionsDialog interface to 
 * reflect the actively selected item in the given listbox element.
 * @param optList The <code>listbox</code> element holding the <code>listitem</code>s
 */
function xsiChangeOptPage(optList)
{
    // Upon page transition store changes of previous page
    xsiStoreOptions();
    
    // Obtain iframe to change the source
    var iFrame = document.getElementById("accessibar-opt-iframe");
    // Set the source to selected items value
    iFrame.setAttribute("src", optList.selectedItem.getAttribute("value"));
    
}

/** 
 * Initializes the options dialog. This function is called when an option page is
 * loaded. It is responsible for calling the relevant initializaion function of the
 * selected page.
 */
function xsiInitOptions()
{
    const selectedPage =
    	document.getElementById("accessibar-opt-list").selectedItem.getAttribute("value");
    // If this is the Keyboard page ~~
    if(selectedPage.indexOf("Keyboard") != -1) {
        xsiInitKeyboardPage();
    }
    // It's the Reader page
    else if(selectedPage.indexOf("Reader") != -1) {
        xsiInitReaderPage(true);
    }
}

/**
 * Initializes the Keyboard options page. 
 */
function xsiInitKeyboardPage()
{
    const pageDoc = document.getElementById("accessibar-opt-iframe").contentDocument;
    var prefService = Components.classes["@mozilla.org/preferences-service;1"]
                      .getService(Components.interfaces.nsIPrefService).getBranch("");
    
    // Set Keyboard Dialog Page Title
    xsiSetPageTitle("xsi-keyboard-listitem");
    
//    xsiSetPrefValue("accessibar.users_guide_key", "char", pageDoc, prefService);
//    xsiSetPrefValue("accessibar.developers_guide_key", "char", pageDoc, prefService);
    xsiSetPrefValue("accessibar.options_key", "char", pageDoc, prefService);
    xsiSetPrefValue("accessibar.font_zoom_in_key", "char", pageDoc, prefService);
    xsiSetPrefValue("accessibar.font_zoom_out_key", "char", pageDoc, prefService);
    xsiSetPrefValue("accessibar.space_inc_key", "char", pageDoc, prefService);
    xsiSetPrefValue("accessibar.space_dec_key", "char", pageDoc, prefService);
    xsiSetPrefValue("accessibar.show_graphics_key", "char", pageDoc, prefService);
    xsiSetPrefValue("accessibar.hide_graphics_key", "char", pageDoc, prefService);
    xsiSetPrefValue("accessibar.reader_on_key", "char", pageDoc, prefService);
    xsiSetPrefValue("accessibar.reader_off_key", "char", pageDoc, prefService);
    xsiSetPrefValue("accessibar.undo_key", "char", pageDoc, prefService);
}

/**
 * Initializes the Reader options page.
 * @param isFirstTime A boolean flag to indicate whether this is an initial initialization
 *        or the page has already been initialized. This enables 
 *        <code>xsiRestoreVoiceDefaults()</code> to utilize this function as well, without
 *        affecting the "Start Reader On Load" option.
 */
function xsiInitReaderPage( isFirstTime )
{
    const pageDoc = document.getElementById("accessibar-opt-iframe").contentDocument;
    var prefService = Components.classes["@mozilla.org/preferences-service;1"]
                      .getService(Components.interfaces.nsIPrefService).getBranch("");
    // Set Reader Dialog Page Title
    xsiSetPageTitle("xsi-reader-listitem");
    
    xsiSetPrefValue("accessibar.TextToSpeech.voice", "char", pageDoc, prefService);
    xsiSetPrefValue("accessibar.TextToSpeech.volume", "int", pageDoc, prefService);
    xsiSetPrefValue("accessibar.TextToSpeech.speed", "int", pageDoc, prefService);
    xsiSetPrefValue("accessibar.TextToSpeech.pitch", "int", pageDoc, prefService);
    xsiSetPrefValue("accessibar.TextToSpeech.range", "int", pageDoc, prefService);
    if(isFirstTime) {
       xsiSetPrefValue("accessibar.TextToSpeech.startReaderOnLoad",
    	               "bool",
    	               pageDoc,
    	               prefService);
    }
}

/**
 * This function restores voice settings to default values.
 */
function xsiRestoreVoiceDefaults()
{   
    xsiStrOpts["accessibar.TextToSpeech.voice"]  = VOICE_DEFAULT;
    xsiIntOpts["accessibar.TextToSpeech.volume"] = VOLUME_DEFAULT;
    xsiIntOpts["accessibar.TextToSpeech.speed"]  = SPEED_DEFAULT;
    xsiIntOpts["accessibar.TextToSpeech.pitch"]  = PITCH_DEFAULT;
    xsiIntOpts["accessibar.TextToSpeech.range"]  = RANGE_DEFAULT;
    
    xsiInitReaderPage(false);
}

/**
 * Sets the preference value to be displayed on the user interface dialog.
 * If the preference is living in the Opts Array, set that value to display on the page,
 * otherwise get the value stored in the user preferences.
 * @param prefName The name of the preference to set.
 * @param prefType The preference type. Acceptable values are "char", "int" and "bool". 
 * @param pageDoc A reference to the <code>contentDocument</code> property of the
 *                <code>iframe</code> option page currently displayed.
 * @param prefService A reference to the root branch of the XPCOM nsIPrefService.
 */
function xsiSetPrefValue( prefName, prefType, pageDoc, prefService )
{
    if( prefType == "char" ) {
        // If the preference is already set
        if(typeof xsiStrOpts[ prefName ] != "undefined") {
            pageDoc.getElementById( prefName ).value = xsiStrOpts[ prefName ];
        }
        else if(prefService.prefHasUserValue( prefName )) {
            pageDoc.getElementById( prefName ).value = prefService.getCharPref(prefName);
        }
    }
    else if( prefType == "int" ) {
        if(typeof xsiIntOpts[ prefName ] != "undefined") {
            pageDoc.getElementById( prefName ).value = xsiIntOpts[ prefName ];
        }
        else if(prefService.prefHasUserValue( prefName )) {
            pageDoc.getElementById( prefName ).value = prefService.getIntPref(prefName);
        }
    }
    else if( prefType == "bool" ) {
        if(typeof xsiBoolOpts[ prefName ] != "undefined") {
            pageDoc.getElementById( prefName ).checked = xsiBoolOpts[ prefName ];
        }
        else if(prefService.prefHasUserValue( prefName )) {
            pageDoc.getElementById(prefName).checked = prefService.getBoolPref(prefName);
        }
    }
}

/**
 * Sets the appropriate string value to display for the <code>dialogheader</code> element.
 * @param listitemId The id of the <code>listitem</code> element where the appropriate 
 *        header string can be found.  
 */
function xsiSetPageTitle( listitemId ) {
    // Set Option Page Title
    var titleElem = document.getElementById("accessibar-opt-header");
    var titleStr = document.getElementById( listitemId ).label;
    
    titleElem.setAttribute("title", titleStr);
}

/**
 *  Stores the user's options in the Opts Arrays to be saved later
 */
function xsiStoreOptions()
{
    const iFrame       = document.getElementById("accessibar-opt-iframe");
    const iFrameSrc    = iFrame.getAttribute("src");
    const pageDoc = iFrame.contentDocument;
    
    // Store Keyboard options in our Opts Arrays.
    if(iFrameSrc.indexOf("Keyboard") != -1) {
//        xsiStrOpts["accessibar.users_guide_key"] = 
//        	pageDoc.getElementById("accessibar.users_guide_key").value;
//        xsiStrOpts["accessibar.developers_guide_key"] = 
//        	pageDoc.getElementById("accessibar.developers_guide_key").value;
        xsiStrOpts["accessibar.options_key"] =
        	pageDoc.getElementById("accessibar.options_key").value;
        xsiStrOpts["accessibar.font_zoom_in_key"] = 
        	pageDoc.getElementById("accessibar.font_zoom_in_key").value;
        xsiStrOpts["accessibar.font_zoom_out_key"] =
        	pageDoc.getElementById("accessibar.font_zoom_out_key").value;
        xsiStrOpts["accessibar.space_inc_key"] = 
        	pageDoc.getElementById("accessibar.space_inc_key").value;
        xsiStrOpts["accessibar.space_dec_key"] = 
        	pageDoc.getElementById("accessibar.space_dec_key").value;
        xsiStrOpts["accessibar.show_graphics_key"] = 
        	pageDoc.getElementById("accessibar.show_graphics_key").value;
        xsiStrOpts["accessibar.hide_graphics_key"] = 
        	pageDoc.getElementById("accessibar.hide_graphics_key").value;
        xsiStrOpts["accessibar.reader_on_key"] = 
        	pageDoc.getElementById("accessibar.reader_on_key").value;
        xsiStrOpts["accessibar.reader_off_key"] = 
        	pageDoc.getElementById("accessibar.reader_off_key").value;
        xsiStrOpts["accessibar.undo_key"] = 
        	pageDoc.getElementById("accessibar.undo_key").value;
        
    }
    // Store Reader options in our Opts Arrays.
    else if(iFrameSrc.indexOf("Reader") != -1) {
        xsiStrOpts["accessibar.TextToSpeech.voice"]  =  
        	pageDoc.getElementById("accessibar.TextToSpeech.voice").value;
        xsiIntOpts["accessibar.TextToSpeech.volume"] = 
        	pageDoc.getElementById("accessibar.TextToSpeech.volume").value;
        xsiIntOpts["accessibar.TextToSpeech.speed"]  = 
        	pageDoc.getElementById("accessibar.TextToSpeech.speed").value;
        xsiIntOpts["accessibar.TextToSpeech.pitch"]  = 
        	pageDoc.getElementById("accessibar.TextToSpeech.pitch").value;
        xsiIntOpts["accessibar.TextToSpeech.range"]  = 
        	pageDoc.getElementById("accessibar.TextToSpeech.range").value;
        xsiBoolOpts["accessibar.TextToSpeech.startReaderOnLoad"] = 
        	pageDoc.getElementById("accessibar.TextToSpeech.startReaderOnLoad").checked;
    }
}

/**
 * Saves the values that have been stored in the Opts Arrays to the nsIPrefService.
 */
function xsiSaveOptions()
{
    var prefService = Components.classes["@mozilla.org/preferences-service;1"]
                      .getService(Components.interfaces.nsIPrefService).getBranch("");
    
    var option      = null;
    var optionValue = null;
    
    // Make sure current page is stored
    xsiStoreOptions();
    
    // Loop through the boolean options
    for(option in xsiBoolOpts) {
        prefService.setBoolPref(option, xsiBoolOpts[option]);
    }
    
    // Loop through the integer options
    for(option in xsiIntOpts) {
        optionValue = xsiIntOpts[option];
        
        // If the option value is set
        if(optionValue) {
            prefService.setIntPref(option, optionValue);
        }
        else if(prefService.prefHasUserValue(option)) {
            prefService.clearUserPref(option);
        }
    }
    
    // Loop through the string options
    for(option in xsiStrOpts) {
        optionValue = xsiStrOpts[option];
        
        // If the option value is set or the preference currently has a value
        if(optionValue || prefService.prefHasUserValue(option)) {
            prefService.setCharPref(option, optionValue);
        }
    }
}